using System;
using System.Collections.Generic;
using System.IO;

/*
 * Class Room - a room in an adventure game.
 *
 * This class is part of the "World of Zuul" application. 
 * "World of Zuul" is a very simple, text based adventure game.  
 *
 * A "Room" represents one location in the scenery of the game.  It is 
 * connected to other rooms via exits.  For each existing exit, the room 
 * stores a reference to the neighboring room.
 * 
 */

public class Room 
{
    private string description;
    private Dictionary<string,Room> exits;        // stores exits of this room.
    private List<Item> items;
//    private Game game;
//    private ArrayList inventory;

    /**
     * Create a room described "description". Initially, it has no exits.
     * "description" is something like "in a kitchen" or "in an open court 
     * yard".
     */
    public Room(string desc) 
    {
        description = desc;
        exits = new Dictionary<string,Room>();
        items = new List<Item>();
    }

    /**
     * Define an exit from this room.
     */
    public void setExit(string direction, Room neighbor) 
    {
        exits.Add(direction, neighbor);
    }

    /**
     * Return the description of the room (the one that was defined in the
     * constructor).
     */
    public virtual string getShortDescription()
    {
        return description;
    }

    /**
     * Return a long description of this room, in the form:
     *     You are in the kitchen.
     *     Exits: north west
     */
    public virtual string getLongDescription()
    {
        string ldesc =  "You are " + getShortDescription() + ".\n" + getExitString();
        if (items.Count > 0) {
            ldesc += "\nThe following things are here:\n";
            for (int n = 0; n < items.Count; n++) {
                Item item = items[n];
                ldesc += "\t" + item.getDesc() + "\n";
            }
        }
        return ldesc;
    }
    
    /**
     * Return a string describing the room's exits, for example
     * "Exits: north west".
     */
    private string getExitString()
    {
        string returnstring = "Exits:";
        //Set<string> keys = exits.KeyCollection();
        foreach (string direction in exits.Keys) {
            returnstring += " " + direction;
        }
        return returnstring;
    }

    /**
     * Return the room that is reached if we go from this room in direction
     * "direction". If there is no room in that direction, return null.
     */
    public virtual Room getExit(string direction) 
    {
        Room newroom;
	exits.TryGetValue(direction, out newroom);
        if (newroom == null) {
            Console.WriteLine("There is no door that way!");
        }
        return newroom;
    }
    
    /**
     * add an item to the room
     */
    public virtual void addItem(Item i) {
        items.Add(i);
    }
    
    /**
     * Get the whole list
     */
    public virtual List<Item> getItems() {
        return items;
    }
    
    public virtual void setDescription(string s) {
        description = s;    // short description
    }
    
    /**
     * respond: take a command & respond to it
     * Note that inventory must be a parameter, as some commands
     * refer to it.
     */
    public virtual Room respond(Command c, List<Item> inventory) {
        string commandWord = c.getCommandWord();
        string dobject = c.getSecondWord();
        if (commandWord == "go") {
            return goRoom(c, inventory);
        } 
        if (commandWord == "take") {
            doTake(c, inventory);
        } else if (commandWord == "drop") {
            doDrop(c, inventory);
        } else {            // pld catchall
            Console.WriteLine("Hmm... I don't seem to know how to "
               + commandWord + " here!");
        }
        return null;    // all cases except goRoom()
    }

    /** 
     * Try to go to one direction. If there is an exit, enter the new
     * room, otherwise print an error message.
     * pld: if getExit() returns null, we can't go that way, and getExit()
     * will print why. Returning a null signals to goRoom *not* to
     * print the description. 
     * Note that there may be cases where getExit returns the same room as the one we're in;
     * the usual understanding of such a situation is that we somehow just returned
     * to the room we were in (as via a teleporter or in a maze), but it wasn't
     * "obviously" the same room.
     */

    private Room goRoom(Command command, List<Item> inv) 
    {
        if(!command.hasSecondWord()) {
            // if there is no second word, we don't know where to go...
            Console.WriteLine("Go where?");
            return null;
        }

        string direction = command.getSecondWord();

        // Try to leave current room.
        Room nextRoom = getExit(direction);

        if (nextRoom == null) {
            Console.WriteLine("There is no door!");
            return null;
        } else {
            Console.WriteLine(nextRoom.getLongDescription());
            return nextRoom;
        }
    }
    
    /**
     * doDrop: 
     *   1. checks if user specified a thing to be dropped
     *   2. checks that it's there in inventory
     *   3. deletes that item from inventory, and adds to currentRoom
     *   4. prints a message about it.
     */
    public void doDrop(Command c, List<Item> inventory) {
        if (! c.hasSecondWord()) {  // "DROP", no object named
            Console.WriteLine("Drop what?");
            return;
        }
        string s = c.getSecondWord();
        Item i = Item.findByName(s, inventory);
        if (i == null) {
            Console.WriteLine("You don't have a " + s);
            return;
        }
        inventory.Remove(i);    
        //currentRoom.addItem(i);
        getItems().Add(i);      // was currentRoom.getItems()
        Console.WriteLine("You have dropped the " + s);
    }
     /**
     * doTake: doesn't do anything yet
     */
    public void doTake(Command c, List<Item> inventory) {
        if (! c.hasSecondWord()) {  // "TAKE", no object named
            Console.WriteLine("Drop what?");
            return;
        }
        string s = c.getSecondWord();
        List<Item> rinventory = getItems();
        Item i = Item.findByName(s, rinventory);
        if (i == null) {
            Console.WriteLine("There isn't a " + s + " here.");
            return;
        }
        rinventory.Remove(i);    
        inventory.Add(i);
        Console.WriteLine("You have taken the " + s);
    }
 

}


